/*  ThinGaMagic2000
 *  version 0.5, October 3rd, 2011
 *
 *  Copyright (C) 2011 Jari Komppa
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty.  In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  2. Altered source versions must be plainly marked as such, and must not be
 *     misrepresented as being the original software.
 *  3. This notice may not be removed or altered from any source distribution.
 *
 * Jari Komppa http://iki.fi/sol/
 *
 *************************************
 * 
 * Note: the code here is not exactly the pinnacle of software design. Could
 * use a refactor or two. But it does more or less what I wanted.
 */

#include "toolkit.h"
#include "AntTweakBar.h"
#include "jansson.h"

void nativecode_init(int parc, char ** pars);
FILE * savefiledialog(const char *title);
FILE * openfiledialog(const char *title);
char * gFilename;
char * gOtherFilename;

// How big a tick difference is considered 'time warp', i.e. skip the time
// (to avoid physics blowing up)
#define TICK_TIMEWARP 1000
// Physics ticks per second
#define PHYSICS_FPS 100

#define USE_PERFCOUNTERS

enum ALLOCSTRATEGY
{
	PREFER_OLD = 0,
	PREFER_NEW,
	FIRST_FIT
};

enum BLENDMODES
{
	BLEND_ALPHA = 0,
	BLEND_ADD,
	BLEND_MUL
};

enum NOISECONTROLLERS
{
	NOISE_DISABLED,
	NOISE_RAND_SHARP,
	NOISE_RAND_LERP,
	NOISE_SIN,
	NOISE_COS,
	NOISE_TRI,
	NOISE_COTRI
};


int tex_font;
int gKeyState = 0;
int gLastTick = 0;
int active_particles = 0;

TwBar *myBar, *barLaunch, *barPhysics, *barDamping, *barEmitter, *barLife, *barSprite, *barTexture;

void handle_key(int keysym, int down)
{
    switch(keysym)
    {
    case SDLK_ESCAPE:
        if (!down)
        {
            exit(0);
        }
        break;
    case SDLK_UP:
    case SDLK_w:
        if (down) gKeyState |= KEY_UP; else gKeyState &= ~KEY_UP;
        break;
    case SDLK_DOWN:
    case SDLK_s:
        if (down) gKeyState |= KEY_DOWN; else gKeyState &= ~KEY_DOWN;
        break;
    case SDLK_LEFT:
    case SDLK_a:
        if (down) gKeyState |= KEY_LEFT; else gKeyState &= ~KEY_LEFT;
        break;
    case SDLK_RIGHT:
    case SDLK_d:
        if (down) gKeyState |= KEY_RIGHT; else gKeyState &= ~KEY_RIGHT;
        break;
    case SDLK_RCTRL:
    case SDLK_LCTRL:
        if (down) gKeyState |= KEY_FIRE; else gKeyState &= ~KEY_FIRE;
        break;
    }
}

void process_events()
{
    SDL_Event event;

    while (SDL_PollEvent(&event)) 
    {
		int handled = TwEventSDL(&event, SDL_MAJOR_VERSION, SDL_MINOR_VERSION);
		
		if (!handled)
        switch (event.type) 
        {
        case SDL_KEYDOWN:
            handle_key(event.key.keysym.sym, 1);
			// If a key is pressed, report it to the widgets
			gUIState.keyentered = event.key.keysym.sym;
			gUIState.keymod = event.key.keysym.mod;
			// if key is ASCII, accept it as character input
			if ((event.key.keysym.unicode & 0xFF80) == 0)
				gUIState.keychar = event.key.keysym.unicode & 0x7f;				
            break;
        case SDL_KEYUP:
            handle_key(event.key.keysym.sym, 0);
            break;
        case SDL_MOUSEMOTION:
			// update mouse position
			gUIState.mousex = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			gUIState.mousey = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            break;
		case SDL_MOUSEBUTTONDOWN:
			// update button down state if left-clicking
			if (event.button.button == 1)
            {
				gUIState.mousedown = 1;
			    gUIState.mousedownx = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			    gUIState.mousedowny = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            }
            if (event.button.button == 4)
            {
                gUIState.scroll = +1;
            }
            if (event.button.button == 5)
            {
                gUIState.scroll = -1;
            }
			break;
		case SDL_MOUSEBUTTONUP:
			// update button down state if left-clicking
			if (event.button.button == 1)
				gUIState.mousedown = 0;
			break;
        case SDL_QUIT:
            SDL_Quit();
            exit(0);
            break;
        case SDL_VIDEORESIZE:
            gScreenWidth = event.resize.w;
            gScreenHeight = event.resize.h;
            initvideo(0);
			TwWindowSize(gScreenWidth, gScreenHeight);
            break;

        }
    }
}

struct Noisedata;

typedef float (*noisecontrollerproc)(int tick, Noisedata &d);

struct Noisedata
{
	noisecontrollerproc func;	
	float r1, r2;
	float period;
	float amplitude;
	float noisetick;
};

struct Particle
{
	float x, y, z;
	float xi, yi, zi;
	float r, ri;
	int alive;
	int life;
	int maxlife;
};


#define MAX_PARTICLES 100000

// These should all be in a struct; kept global for simplicity of
// the editor. Needs to be refactored in order to be useful..

Particle particle[MAX_PARTICLES];

float yrot = 0;
float camy = 100;
float camz = -200;
float aty = 40;

float life_base = 2000;
int   life_noise = NOISE_DISABLED;
Noisedata life_noisedata;

int pretick = 0;

float xi_base = 0;
int   xi_noise = NOISE_DISABLED;
Noisedata xi_noisedata;
float yi_base = 0;
int   yi_noise = NOISE_DISABLED;
Noisedata yi_noisedata;
float zi_base = 0;
int   zi_noise = NOISE_DISABLED;
Noisedata zi_noisedata;

float xg_base = 0;
int   xg_noise = NOISE_DISABLED;
Noisedata xg_noisedata;
float yg_base = 0;
int   yg_noise = NOISE_DISABLED;
Noisedata yg_noisedata;
float zg_base = 0;
int   zg_noise = NOISE_DISABLED;
Noisedata zg_noisedata;
float dampx = 0.05;
float dampy = 0.05;
float dampz = 0.05;

float startx = 0;
float starty = 40;
float startz = 0;

float xsize = 0;
float ysize = 0;
float zsize = 0;

float size0 = 2, size1 = 2;
float alpha0 = 1, alpha1 = 1;

float color0[3]={1,1,1}, color1[3]={1,1,1};

float r_base = 0;
int   r_noise = NOISE_DISABLED;
Noisedata r_noisedata;
float ri_base = 0;
int   ri_noise = NOISE_DISABLED;
Noisedata ri_noisedata;

float emithz_base = 100;
int   emithz_noise = NOISE_DISABLED;
Noisedata emithz_noisedata;
float emitcount_base = 1;
int   emitcount_noise = NOISE_DISABLED;
Noisedata emitcount_noisedata;
float emitbuf = 0;

int particle_max = 1000;
int alloc_strategy = PREFER_OLD;

int blendmode = BLEND_ALPHA;
int zbufwrite = 1;
int sorted = 0;

int tex_particle;
int tex_frames;

char *nt = "<no texture>";

char *texture_fn = nt;
char *texture_fn_short = nt;

float noise_disabled(int tick, Noisedata &d)
{
	return 0;
}

float noise_hard(int tick, Noisedata &d)
{
	return (gVisualRand.genrand_real1() - 0.5f) * 2 * d.amplitude;
}

float noise_sin(int tick, Noisedata &d)
{
	return sin(6.28 * (tick / d.period)) * d.amplitude;
}

float noise_cos(int tick, Noisedata &d)
{
	return cos(6.28 * (tick / d.period)) * d.amplitude;
}

float noise_cotriangle(int tick, Noisedata &d)
{
	float discard;
	return (abs(modf(tick / d.period,&discard) - 0.5f)*4-1.0f) * d.amplitude;
}

float noise_triangle(int tick, Noisedata &d)
{
	float discard;
	return (abs(modf(tick / d.period + 0.25f, &discard) - 0.5f)*4.0f-1.0f) * d.amplitude;
}

float noise_sharprand(int tick, Noisedata &d)
{
	float t;
	float f = modf(tick / d.period, &t);
	if (t != d.noisetick)
	{
		d.noisetick = t;
		d.r1 = (gVisualRand.genrand_real1() - 0.5f) * 2 * d.amplitude;
	}	
	return d.r1;
}

#define SMOOTHSTEP(x) ((x) * (x) * (3 - 2 * (x)))

float noise_lerprand(int tick, Noisedata &d)
{
	float t;
	float f = modf(tick / d.period, &t);
	f = SMOOTHSTEP(f);
	if (t != d.noisetick)
	{
		d.noisetick = t;
		d.r1 = d.r2;
		d.r2 = (gVisualRand.genrand_real1() - 0.5f) * 2 * d.amplitude;
	}	
	return d.r2 * f + d.r1 * (1 - f);
}

void setcontroller(Noisedata &d, int noise)
{
	switch (noise)
	{
	case NOISE_DISABLED:
		d.func = noise_disabled;
		break;
	case NOISE_RAND_SHARP:
		d.func = noise_sharprand;
		break;
	case NOISE_RAND_LERP:
		d.func = noise_lerprand;
		break;
	case NOISE_SIN:
		d.func = noise_sin;
		break;
	case NOISE_COS:
		d.func = noise_cos;
		break;
	case NOISE_TRI:
		d.func = noise_triangle;
		break;
	case NOISE_COTRI:
		d.func = noise_cotriangle;
		break;
	default:
		d.func = noise_hard;
	}

	if (d.period == 0 && noise != NOISE_DISABLED)
		d.func = noise_hard;
}

void spawn_particle(int tick)
{
	if (particle_max == 0)
		return;
	int i = gVisualRand.genrand_int31() % particle_max;
	if (alloc_strategy == PREFER_OLD && particle[i].alive)
	{
		return;
	}
	else
	if (alloc_strategy == FIRST_FIT)
	{
		i = 0;
		while (particle[i].alive && i < particle_max) i++;
		if (i == particle_max)
			return;
	}
	
	if (particle[i].alive && alloc_strategy)
		return;
	particle[i].alive = 1;
	active_particles++;
	particle[i].life = life_noisedata.func(tick, life_noisedata) + life_base;
	if (particle[i].life < 0) 
	{
		particle[i].alive = 0;
		active_particles--;
		return;
	}

	particle[i].maxlife = particle[i].life;
	particle[i].x = startx + (gVisualRand.genrand_real1() - 0.5f) * 2 * xsize;
	particle[i].y = starty + (gVisualRand.genrand_real1() - 0.5f) * 2 * ysize;
	particle[i].z = startz + (gVisualRand.genrand_real1() - 0.5f) * 2 * zsize;	
	particle[i].xi = xi_noisedata.func(tick, xi_noisedata) + xi_base;
	particle[i].yi = yi_noisedata.func(tick, yi_noisedata) + yi_base;
	particle[i].zi = zi_noisedata.func(tick, zi_noisedata) + zi_base;

	particle[i].r = r_noisedata.func(tick, r_noisedata) + r_base;
	particle[i].ri = ri_noisedata.func(tick, ri_noisedata) + ri_base;
}

void physics_tick(int timeslice, int tick)
{
	int i;

	float t = timeslice * 0.001; // turn ms into s

	// these only need to be done if the controller changes,
	// but we don't know when that happens.. so.. let's do it once
	// per physics tick to be sure.
	// When actually using a particle system this only needs to 
	// be done once.
	setcontroller(xi_noisedata, xi_noise);
	setcontroller(yi_noisedata, yi_noise);
	setcontroller(zi_noisedata, zi_noise);
	setcontroller(xg_noisedata, xg_noise);
	setcontroller(yg_noisedata, yg_noise);
	setcontroller(zg_noisedata, zg_noise);
	setcontroller(r_noisedata, r_noise);
	setcontroller(ri_noisedata, ri_noise);
	setcontroller(life_noisedata, life_noise);
	setcontroller(emithz_noisedata, emithz_noise);
	setcontroller(emitcount_noisedata, emitcount_noise);

	if (gUIState.mousedown)
	{
		if (gKeyState & KEY_FIRE)
		{
			startx += (gUIState.mousedownx - gUIState.mousex) / 2;
			gUIState.mousedownx = gUIState.mousex;
			starty += (gUIState.mousedowny - gUIState.mousey) / 2;
			gUIState.mousedowny = gUIState.mousey;
		}
		else
		{
			yrot -= (gUIState.mousedownx - gUIState.mousex) / 4.0f;
			gUIState.mousedownx = gUIState.mousex;
			camz += (gUIState.mousedowny - gUIState.mousey);
			gUIState.mousedowny = gUIState.mousey;
			if (camz > -10)
				camz = -10;
		}
	}

// for optimizing the render..
//	if (active_particles > 95000)		return;


	float emit_inc = (timeslice * (emithz_noisedata.func(tick, emithz_noisedata) + emithz_base)) / 1000.0f;
	if (emit_inc > 0)
		emitbuf += emit_inc;

	while (emitbuf > 1)
	{
		int emitcount = (int)floor(emitcount_noisedata.func(tick, emitcount_noisedata) + emitcount_base);
		for (i = 0; i < emitcount; i++)
			spawn_particle(tick);
		emitbuf--;
	}

	for (i = 0; i < MAX_PARTICLES; i++)
	{
		if (particle[i].alive)
		{
			particle[i].life -= timeslice;
			if (particle[i].life <= 0)
			{
				particle[i].alive = 0;
				active_particles--;
			}
			else
			{
				float forcex = xg_noisedata.func(tick, xg_noisedata) + xg_base - (particle[i].xi * dampx);
				float forcey = yg_noisedata.func(tick, yg_noisedata) + yg_base - (particle[i].yi * dampy);
				float forcez = zg_noisedata.func(tick, zg_noisedata) + zg_base - (particle[i].zi * dampz);
				
				particle[i].x += particle[i].xi * t + 0.5 * forcex * t * t;
				particle[i].y += particle[i].yi * t + 0.5 * forcey * t * t;
				particle[i].z += particle[i].zi * t + 0.5 * forcez * t * t;

				particle[i].xi += forcex * t;
				particle[i].yi += forcey * t;
				particle[i].zi += forcez * t;

				if (particle[i].y < 0)
				{
					particle[i].y = -particle[i].y;
					particle[i].yi = -particle[i].yi;
				}

				particle[i].r += particle[i].ri;
			}
		}
	}
}

void set3d()
{
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(60,1,10,1000);

	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	gluLookAt(0,camy,camz,0,aty,0,0,1,0);

	glRotatef(yrot, 0, 1, 0);
}

float *vertdata = 0;
float *colordata;
float *uvdata;

void drawparticles(float ry)
{
	int i;

	if (vertdata == 0)
	{
		vertdata  = new float[MAX_PARTICLES * 3 * 4];
		colordata = new float[MAX_PARTICLES * 4 * 4];
		uvdata    = new float[MAX_PARTICLES * 2 * 4];
	}

	float morig[4*4];

	glGetFloatv(GL_MODELVIEW_MATRIX, &morig[0]);

	int idx = 0;

	for (i = 0; i < MAX_PARTICLES; i++)
	{
		if (particle[i].alive)
		{
			float right[3], up[3];
			float cs = cos(particle[i].r);
			float sn = sin(particle[i].r);

			float age = particle[i].life / (float)particle[i].maxlife;

			float frameht = 1.0 / tex_frames;
			float frameofs = floor((1-age) * tex_frames) * frameht;			
			
			float x = particle[i].x;
			float y = particle[i].y * ry; // for the mirror. Yes, we do that much extra work.
			float z = particle[i].z;

			float s = size0 * age + size1 * (1-age);

			// rotation matrix multiply written out and optimized (plus scaling)

			right[0] = (morig[0 * 4 + 0] * cs + morig[0 * 4 + 1] * -sn) * s;
			right[1] = (morig[1 * 4 + 0] * cs + morig[1 * 4 + 1] * -sn) * s;
			right[2] = (morig[2 * 4 + 0] * cs + morig[2 * 4 + 1] * -sn) * s;
			up[0]    = (morig[0 * 4 + 0] * sn + morig[0 * 4 + 1] * cs)  * s;
			up[1]    = (morig[1 * 4 + 0] * sn + morig[1 * 4 + 1] * cs)  * s;
			up[2]    = (morig[2 * 4 + 0] * sn + morig[2 * 4 + 1] * cs)  * s;

			vertdata[(idx * 4 + 0) * 3 + 0] = x - (right[0] + up[0]);
			vertdata[(idx * 4 + 0) * 3 + 1] = y - (right[1] + up[1]);
			vertdata[(idx * 4 + 0) * 3 + 2] = z - (right[2] + up[2]);					
			
			vertdata[(idx * 4 + 1) * 3 + 0] = x + (right[0] - up[0]);
			vertdata[(idx * 4 + 1) * 3 + 1] = y + (right[1] - up[1]);
			vertdata[(idx * 4 + 1) * 3 + 2] = z + (right[2] - up[2]);

			vertdata[(idx * 4 + 2) * 3 + 0] = x + (right[0] + up[0]);
			vertdata[(idx * 4 + 2) * 3 + 1] = y + (right[1] + up[1]);
			vertdata[(idx * 4 + 2) * 3 + 2] = z + (right[2] + up[2]);
			
			vertdata[(idx * 4 + 3) * 3 + 0] = x - (right[0] - up[0]);
			vertdata[(idx * 4 + 3) * 3 + 1] = y - (right[1] - up[1]);
			vertdata[(idx * 4 + 3) * 3 + 2] = z - (right[2] - up[2]);

			uvdata[(idx * 4 + 0) * 2 + 0] = 0;
			uvdata[(idx * 4 + 0) * 2 + 1] = frameofs+frameht;

			uvdata[(idx * 4 + 1) * 2 + 0] = 1;
			uvdata[(idx * 4 + 1) * 2 + 1] = frameofs+frameht;

			uvdata[(idx * 4 + 2) * 2 + 0] = 1;
			uvdata[(idx * 4 + 2) * 2 + 1] = frameofs;

			uvdata[(idx * 4 + 3) * 2 + 0] = 0;
			uvdata[(idx * 4 + 3) * 2 + 1] = frameofs;

			float alpha = alpha0 * age + alpha1 * (1-age);
			float red = color0[0] * age + color1[0] * (1-age);
			float grn = color0[1] * age + color1[1] * (1-age);
			float blu = color0[2] * age + color1[2] * (1-age);
			
			colordata[(idx * 4 + 0) * 4 + 0] = red;
			colordata[(idx * 4 + 0) * 4 + 1] = grn;
			colordata[(idx * 4 + 0) * 4 + 2] = blu;
			colordata[(idx * 4 + 0) * 4 + 3] = alpha;

			colordata[(idx * 4 + 1) * 4 + 0] = red;
			colordata[(idx * 4 + 1) * 4 + 1] = grn;
			colordata[(idx * 4 + 1) * 4 + 2] = blu;
			colordata[(idx * 4 + 1) * 4 + 3] = alpha;

			colordata[(idx * 4 + 2) * 4 + 0] = red;
			colordata[(idx * 4 + 2) * 4 + 1] = grn;
			colordata[(idx * 4 + 2) * 4 + 2] = blu;
			colordata[(idx * 4 + 2) * 4 + 3] = alpha;

			colordata[(idx * 4 + 3) * 4 + 0] = red;
			colordata[(idx * 4 + 3) * 4 + 1] = grn;
			colordata[(idx * 4 + 3) * 4 + 2] = blu;
			colordata[(idx * 4 + 3) * 4 + 3] = alpha;
						
			idx++;			
		}
	}

	// data gathered, let's render!

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, tex_particle);
	
	switch (blendmode)
	{
	case BLEND_ADD:
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		break;
	case BLEND_ALPHA:
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		break;
	case BLEND_MUL:
		glBlendFunc(GL_DST_COLOR, GL_ONE_MINUS_SRC_ALPHA);
		break;
	}

	if (zbufwrite)
	{
		glDepthMask(GL_TRUE);
	}
	else
	{
		glDepthMask(GL_FALSE);
	}

	glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, vertdata);
	glTexCoordPointer(2, GL_FLOAT, 0, uvdata);
	glColorPointer(4,GL_FLOAT,0,colordata);
	glDrawArrays(GL_QUADS,0,idx*4);
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);

	glDisable(GL_TEXTURE_2D);

	glDepthMask(GL_TRUE);

}

static void draw_screen_ingame()
{
    int tick = SDL_GetTicks();

    int i;    

    if (tick - gLastTick > TICK_TIMEWARP) 
        gLastTick = tick;

    if (gLastTick >= tick)
    {
        SDL_Delay(1);
        return;
    }

    while (gLastTick < tick)
    {
		
		physics_tick(1000 / PHYSICS_FPS, gLastTick);

        gLastTick += 1000 / PHYSICS_FPS;
		
		/*
		// for variable timestep.. useful for profiling (with very sleepy for instance)
		physics_tick(tick - gLastTick);
		gLastTick = tick;
		*/
    }

    ////////////////////////////////////
    // Rendering
    ////////////////////////////////////

    glClearColor(0,0,0,1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0.2);

    ////////////////////////////////////
    // testcode
    ////////////////////////////////////

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    ////////////////////////////////////
	// Let's do some 3d..
    ////////////////////////////////////

	set3d();

	glEnable(GL_POINT_SMOOTH);
	glPointSize(10);

	drawparticles(-1);

	// draw a plane
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(0,0,0.3,0.75);
    glBegin(GL_TRIANGLE_STRIP);
      glVertex3f(-250,0,-250);
      glVertex3f(-250,0,250);
      glVertex3f(250,0,-250);
      glVertex3f(250,0,250);
    glEnd();

	// draw grid of lines
	glEnable(GL_LINE_SMOOTH);
	glDisable(GL_DEPTH_TEST);
	glBegin(GL_LINES);
	for (i = 0; i < 51; i++)
	{
		glColor4f(0,1.0 - abs(i-25)/25.0f,0,1.0);

		glVertex3f((i-25)*10,0,-25*10);
		glVertex3f((i-25)*10,0,25*10);
		glVertex3f(-25*10,0,(i-25)*10);
		glVertex3f(25*10,0,(i-25)*10);
	}
	glEnd();
	glEnable(GL_DEPTH_TEST);

	drawparticles(1);

	glDisable(GL_DEPTH_TEST);

    ////////////////////////////////////
	// Back to 2d
    ////////////////////////////////////

	set2d();

	char temp[256];
	sprintf(temp, "active particles: %d", active_particles);
    quickfont_drawstring(tex_font, temp, 320, 16, 0xffffff, 1, 1);

#ifdef USE_PERFCOUNTERS

	render_perfcounters(tex_font);

#endif

	TwDraw();
    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}



void draw_screen()
{
    draw_screen_ingame();
}

void TW_CALL resetparticles(void *clientdata)
{
	int i;
	for (i = 0; i < MAX_PARTICLES; i++)
		particle[i].alive = 0;

	active_particles = 0;

	for (i = 0; i < pretick; i++)
		physics_tick(1000 / PHYSICS_FPS, i * (1000 / PHYSICS_FPS));

	gLastTick = SDL_GetTicks();
}

void TW_CALL reloadtx(void *clientdata)
{
	tex_particle = load_texture_forcereload(texture_fn);
	tex_frames = last_tex_h / last_tex_w;	
}


void TW_CALL loadtx(void *clientdata)
{
	FILE * f = openfiledialog("Load texture");
	if (f)
	{
		fclose(f);
		if (texture_fn != nt)
			delete[] texture_fn;
		texture_fn = gOtherFilename;
		texture_fn_short = strrchr(texture_fn,'/');
		if (texture_fn_short == NULL)
			texture_fn_short = strrchr(texture_fn,'\\');
		else
			texture_fn_short++;
		if (texture_fn_short == NULL)
			texture_fn_short = texture_fn;
		else
			texture_fn_short++;

		gOtherFilename = NULL;
		reloadtx(NULL);
	}
}

void TW_CALL savepr4(void *clientdata)
{
	FILE * f = savefiledialog(NULL);
	if (f)
	{
		fclose(f);
		json_set_alloc_funcs(malloc, free);
		json_t *root = json_object();
		json_object_set(root,"generator",json_string(TITLE));
		json_object_set(root,"fileversion",json_integer(1));
		json_object_set(root,"life_base",json_real(life_base));
		json_object_set(root,"life_noise",json_integer(life_noise));
		json_object_set(root,"life_noise.period",json_real(life_noisedata.period));
		json_object_set(root,"life_noise.amplitude",json_real(life_noisedata.amplitude));
		json_object_set(root,"pretick",json_integer(pretick));
		json_object_set(root,"xi_base",json_real(xi_base));
		json_object_set(root,"xi_noise",json_integer(xi_noise));
		json_object_set(root,"xi_noise.period",json_real(xi_noisedata.period));
		json_object_set(root,"xi_noise.amplitude",json_real(xi_noisedata.amplitude));
		json_object_set(root,"yi_base",json_real(yi_base));
		json_object_set(root,"yi_noise",json_integer(yi_noise));
		json_object_set(root,"yi_noise.period",json_real(yi_noisedata.period));
		json_object_set(root,"yi_noise.amplitude",json_real(yi_noisedata.amplitude));
		json_object_set(root,"zi_base",json_real(zi_base));
		json_object_set(root,"zi_noise",json_integer(zi_noise));
		json_object_set(root,"zi_noise.period",json_real(zi_noisedata.period));
		json_object_set(root,"zi_noise.amplitude",json_real(zi_noisedata.amplitude));
		json_object_set(root,"xg_base",json_real(xg_base));
		json_object_set(root,"xg_noise",json_integer(xg_noise));
		json_object_set(root,"xg_noise.period",json_real(xg_noisedata.period));
		json_object_set(root,"xg_noise.amplitude",json_real(xg_noisedata.amplitude));
		json_object_set(root,"yg_base",json_real(yg_base));
		json_object_set(root,"yg_noise",json_integer(yg_noise));
		json_object_set(root,"yg_noise.period",json_real(yg_noisedata.period));
		json_object_set(root,"yg_noise.amplitude",json_real(yg_noisedata.amplitude));
		json_object_set(root,"zg_base",json_real(zg_base));
		json_object_set(root,"zg_noise",json_integer(zg_noise));
		json_object_set(root,"zg_noise.period",json_real(zg_noisedata.period));
		json_object_set(root,"zg_noise.amplitude",json_real(zg_noisedata.amplitude));
		json_object_set(root,"dampx",json_real(dampx));
		json_object_set(root,"dampy",json_real(dampy));
		json_object_set(root,"dampz",json_real(dampz));
		json_object_set(root,"emitter_xsize",json_real(xsize));
		json_object_set(root,"emitter_ysize",json_real(ysize));
		json_object_set(root,"emitter_zsize",json_real(zsize));
		json_object_set(root,"size0",json_real(size0));
		json_object_set(root,"size1",json_real(size1));
		json_object_set(root,"alpha0",json_real(alpha0));
		json_object_set(root,"alpha1",json_real(alpha1));
		json_object_set(root,"red0",json_real(color0[0]));
		json_object_set(root,"red1",json_real(color1[0]));
		json_object_set(root,"grn0",json_real(color0[1]));
		json_object_set(root,"grn1",json_real(color1[1]));
		json_object_set(root,"blu0",json_real(color0[2]));
		json_object_set(root,"blu1",json_real(color1[2]));
		json_object_set(root,"r_base",json_real(r_base));
		json_object_set(root,"r_noise",json_integer(r_noise));
		json_object_set(root,"r_noise.period",json_real(r_noisedata.period));
		json_object_set(root,"r_noise.amplitude",json_real(r_noisedata.amplitude));
		json_object_set(root,"ri_base",json_real(ri_base));
		json_object_set(root,"ri_noise",json_integer(ri_noise));
		json_object_set(root,"ri_noise.period",json_real(ri_noisedata.period));
		json_object_set(root,"ri_noise.amplitude",json_real(ri_noisedata.amplitude));
		json_object_set(root,"emithz_base",json_real(emithz_base));
		json_object_set(root,"emithz_noise",json_integer(emithz_noise));
		json_object_set(root,"emithz_noise.period",json_real(emithz_noisedata.period));
		json_object_set(root,"emithz_noise.amplitude",json_real(emithz_noisedata.amplitude));
		json_object_set(root,"emitcount_base",json_real(emitcount_base));
		json_object_set(root,"emitcount_noise",json_integer(emitcount_noise));
		json_object_set(root,"emitcount_noise.period",json_real(emitcount_noisedata.period));
		json_object_set(root,"emitcount_noise.amplitude",json_real(emitcount_noisedata.amplitude));
		json_object_set(root,"max_particles",json_integer(particle_max));
		json_object_set(root,"allocstrategy",json_integer(alloc_strategy));
		json_object_set(root,"blendmode",json_integer(blendmode));
		json_object_set(root,"zwrite",json_integer(zbufwrite));
		json_object_set(root,"texture",json_string(texture_fn));
		json_object_set(root,"texture_short",json_string(texture_fn_short));
		json_object_set(root,"texture_frames",json_integer(tex_frames));

		json_dump_file(root, gFilename, JSON_INDENT(2) | JSON_ENSURE_ASCII | JSON_PRESERVE_ORDER);
		json_decref(root);
	}
}

static float json_getfloat(json_t *parent, const char *key)
{
	json_t * v = json_object_get(parent, key);
	return json_real_value(v);
}

static int json_getint(json_t *parent, const char *key)
{
	json_t * v = json_object_get(parent, key);
	return json_integer_value(v);
}

static const char * json_getstring(json_t *parent, const char *key)
{
	json_t * v = json_object_get(parent, key);
	return json_string_value(v);
}


void TW_CALL loadpr4(void *clientdata)
{
	FILE * f = openfiledialog(NULL);
	if (f)
	{
		fclose(f);
		json_error_t err;
		json_t * root = json_load_file(gFilename, JSON_DISABLE_EOF_CHECK, &err);
		if (root == NULL)
			return;

		//"generator",json_string(TITLE));
		//"fileversion",json_integer(1));
		life_base = json_getfloat(root, "life_base");
		life_noise = json_getint(root, "life_noise");
		life_noisedata.period = json_getfloat(root, "life_noise.period");
		life_noisedata.amplitude = json_getfloat(root, "life_noise.amplitude");
		pretick = json_getint(root, "pretick");
		xi_base = json_getfloat(root, "xi_base");
		xi_noise = json_getint(root, "xi_noise");
		xi_noisedata.period = json_getfloat(root, "xi_noise.period");
		xi_noisedata.amplitude = json_getfloat(root, "xi_noise.amplitude");
		yi_base = json_getfloat(root, "yi_base");
		yi_noise = json_getint(root, "yi_noise");
		yi_noisedata.period = json_getfloat(root, "yi_noise.period");
		yi_noisedata.amplitude = json_getfloat(root, "yi_noise.amplitude");
		zi_base = json_getfloat(root, "zi_base");
		zi_noise = json_getint(root, "zi_noise");
		zi_noisedata.period = json_getfloat(root, "zi_noise.period");
		zi_noisedata.amplitude = json_getfloat(root, "zi_noise.amplitude");
		xg_base = json_getfloat(root, "xg_base");
		xg_noise = json_getint(root, "xg_noise");
		xg_noisedata.period = json_getfloat(root, "xg_noise.period");
		xg_noisedata.amplitude = json_getfloat(root, "xg_noise.amplitude");
		yg_base = json_getfloat(root, "yg_base");
		yg_noise = json_getint(root, "yg_noise");
		yg_noisedata.period = json_getfloat(root, "yg_noise.period");
		yg_noisedata.amplitude = json_getfloat(root, "yg_noise.amplitude");
		zg_base = json_getfloat(root, "zg_base");
		zg_noise = json_getint(root, "zg_noise");
		zg_noisedata.period = json_getfloat(root, "zg_noise.period");
		zg_noisedata.amplitude = json_getfloat(root, "zg_noise.amplitude");
		dampx = json_getfloat(root, "dampx");
		dampy = json_getfloat(root, "dampy");
		dampz = json_getfloat(root, "dampz");
		xsize = json_getfloat(root, "emitter_xsize");
		ysize = json_getfloat(root, "emitter_ysize");
		zsize = json_getfloat(root, "emitter_zsize");
		size0 = json_getfloat(root, "size0");
		size1 = json_getfloat(root, "size1");
		alpha0 = json_getfloat(root, "alpha0");
		alpha1 = json_getfloat(root, "alpha1");
		color0[0] = json_getfloat(root, "red0");
		color1[0] = json_getfloat(root, "red1");
		color0[1] = json_getfloat(root, "grn0");
		color1[1] = json_getfloat(root, "grn1");
		color0[2] = json_getfloat(root, "blu0");
		color1[2] = json_getfloat(root, "blu1");
		r_base = json_getfloat(root, "r_base");
		r_noise = json_getint(root, "r_noise");
		r_noisedata.period = json_getfloat(root, "r_noise.period");
		r_noisedata.amplitude = json_getfloat(root, "r_noise.amplitude");
		ri_base = json_getfloat(root, "ri_base");
		ri_noise = json_getint(root, "ri_noise");
		ri_noisedata.period = json_getfloat(root, "ri_noise.period");
		ri_noisedata.amplitude = json_getfloat(root, "ri_noise.amplitude");
		emithz_base = json_getfloat(root, "emithz_base");
		emithz_noise = json_getint(root, "emithz_noise");
		emithz_noisedata.period = json_getfloat(root, "emithz_noise.period");
		emithz_noisedata.amplitude = json_getfloat(root, "emithz_noise.amplitude");
		emitcount_base = json_getfloat(root, "emitcount_base");
		emitcount_noise = json_getint(root, "emitcount_noise");
		emitcount_noisedata.period = json_getfloat(root, "emitcount_noise.period");
		emitcount_noisedata.amplitude = json_getfloat(root, "emitcount_noise.amplitude");
		particle_max = json_getint(root, "max_particles");
		alloc_strategy = json_getint(root, "allocstrategy");
		blendmode = json_getint(root, "blendmode");
		zbufwrite = json_getint(root, "zwrite");
		tex_frames = json_getint(root, "texture_frames");
		if (texture_fn && texture_fn != nt)
			delete[] texture_fn;
		const char * t = json_getstring(root, "texture");
		FILE * f = fopen(t, "rb");
		if (f)
		{
			fclose(f);
		}
		else
		{
			t = json_getstring(root,"texture_short");
		}
		int l = strlen(t);
		texture_fn = new char[l+1];
		texture_fn[l] = 0;
		memcpy(texture_fn, t, l);
		texture_fn_short = strrchr(texture_fn, '/');
		if (!texture_fn_short)
			texture_fn_short = strrchr(texture_fn, '\\');
		if (texture_fn_short)
			texture_fn_short++;
		else
			texture_fn_short = texture_fn;

		reloadtx(NULL);
		resetparticles(NULL);

		json_decref(root);
	}
}


void initbars()
{
	TwEnumVal allocstrategyEV[] = 
	{
		{ PREFER_OLD, "Prefer old"}, 
		{ PREFER_NEW, "Prefer new"}, 
		{ FIRST_FIT, "First fit"}
	};
	TwEnumVal noisecontrollerEV[] =
	{
		{ NOISE_DISABLED,   "Disabled" },
		{ NOISE_RAND_SHARP, "Random (sharp)" },
		{ NOISE_RAND_LERP,  "Random (interpolated)" },
		{ NOISE_SIN,        "Sine wave" },
		{ NOISE_COS,        "Cosine wave" },
		{ NOISE_TRI,        "Triangle wave" },
		{ NOISE_COTRI,      "Cotriangle wave" }
	};
	TwEnumVal blendmodeEV[] = 
	{
		{ BLEND_ALPHA, "Alpha blend"}, 
		{ BLEND_ADD, "Additive blend"}, 
		{ BLEND_MUL, "Multiplicative blend"}
	};



	TwType allocstrategy, noisecontroller, blendmodes;

	allocstrategy = TwDefineEnum("AllocStrategy", allocstrategyEV, 3);
	noisecontroller = TwDefineEnum("NoiseController", noisecontrollerEV, 7);
	blendmodes = TwDefineEnum("BlendMode", blendmodeEV, 3);

	myBar = TwNewBar("System Controls");
	barLaunch = TwNewBar("Launch Params");
	barPhysics = TwNewBar("Physics Params");
	barEmitter = TwNewBar("Emitter Params");
	barLife = TwNewBar("Life Params");
	barSprite = TwNewBar("Geometry params");
	barTexture = TwNewBar("Texture params");

	unsigned int truetrue=1;
	//TwSetParam(myBar,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barLaunch,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barPhysics,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barEmitter,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barLife,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barSprite,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);
	TwSetParam(barTexture,NULL,"iconified", TW_PARAM_INT32, 1, &truetrue);

	TwAddVarRW(myBar, "Camera rotation", TW_TYPE_FLOAT, &yrot, NULL);
	TwAddVarRW(myBar, "Camera height", TW_TYPE_FLOAT, &camy, NULL);
	TwAddVarRW(myBar, "Camera distance", TW_TYPE_FLOAT, &camz, " min=-500 max=-10 ");
	TwAddVarRW(myBar, "Target height", TW_TYPE_FLOAT, &aty, NULL);
	TwAddSeparator(myBar, NULL, NULL);
	TwAddButton(myBar, "Save system...", savepr4, NULL, NULL);
	TwAddButton(myBar, "Load system...", loadpr4, NULL, NULL);

	TwAddButton(barLife, "Reset particle system", resetparticles, NULL, NULL);// " label='Reset' ");
	TwAddSeparator(barLife,NULL,NULL);
	TwAddVarRW(barLife, "Pre-tick ", TW_TYPE_INT32, &pretick, " min=0 max=10000 ");
	TwAddSeparator(barLife,NULL,NULL);
	TwAddVarRW(barLife, "Life base", TW_TYPE_FLOAT, &life_base, " min=0 max=10000  ");
	TwAddVarRW(barLife, "Life noise", noisecontroller, &life_noise, NULL);
	TwAddVarRW(barLife, "Life amp", TW_TYPE_FLOAT, &life_noisedata.amplitude, " step=1 ");
	TwAddVarRW(barLife, "Life period", TW_TYPE_FLOAT, &life_noisedata.period, " step=1 ");
	TwAddSeparator(barLife,NULL,NULL);
	TwAddVarRW(barLife, "Particle count cap", TW_TYPE_INT32, &particle_max, " step=10 min=0 max=100000  ");
	TwAddVarRW(barLife, "Alloc strategy", allocstrategy, &alloc_strategy, NULL);
	TwAddSeparator(barLife,NULL,NULL);
	TwAddVarRW(barLife, "Emit Hz base", TW_TYPE_FLOAT, &emithz_base, " step=10 min=0 max=1000000  ");
	TwAddVarRW(barLife, "Emit Hz noise", noisecontroller, &emithz_noise, NULL);
	TwAddVarRW(barLife, "Emit Hz amp", TW_TYPE_FLOAT, &emithz_noisedata.amplitude, " step=10 min=0 max=1000000  ");
	TwAddVarRW(barLife, "Emit Hz period", TW_TYPE_FLOAT, &emithz_noisedata.period, " step=1 ");

	TwAddVarRW(barLife, "Emit # base", TW_TYPE_FLOAT, &emitcount_base, " step=1 min=0 max=1000  ");
	TwAddVarRW(barLife, "Emit # noise", noisecontroller, &emitcount_noise, NULL);
	TwAddVarRW(barLife, "Emit # amp", TW_TYPE_FLOAT, &emitcount_noisedata.amplitude, " step=1 min=0 max=1000  ");
	TwAddVarRW(barLife, "Emit # period", TW_TYPE_FLOAT, &emitcount_noisedata.period, " step=1 ");

	TwAddVarRW(barLaunch, "Xi base", TW_TYPE_FLOAT, &xi_base, " step=0.01 ");
	TwAddVarRW(barLaunch, "Xi noise", noisecontroller, &xi_noise, NULL);
	TwAddVarRW(barLaunch, "Xi amp", TW_TYPE_FLOAT, &xi_noisedata.amplitude, " step=0.01 ");
	TwAddVarRW(barLaunch, "Xi period", TW_TYPE_FLOAT, &xi_noisedata.period, " step=1 ");
	TwAddVarRW(barLaunch, "Yi base", TW_TYPE_FLOAT, &yi_base, " step=0.01 ");
	TwAddVarRW(barLaunch, "Yi noise", noisecontroller, &yi_noise, NULL);
	TwAddVarRW(barLaunch, "Yi amp", TW_TYPE_FLOAT, &yi_noisedata.amplitude, " step=0.01 ");
	TwAddVarRW(barLaunch, "Yi period", TW_TYPE_FLOAT, &yi_noisedata.period, " step=1 ");
	TwAddVarRW(barLaunch, "Zi base", TW_TYPE_FLOAT, &zi_base, " step=0.01 ");
	TwAddVarRW(barLaunch, "Zi noise", noisecontroller, &zi_noise, NULL);
	TwAddVarRW(barLaunch, "Zi amp", TW_TYPE_FLOAT, &zi_noisedata.amplitude, " step=0.01 ");
	TwAddVarRW(barLaunch, "Zi period", TW_TYPE_FLOAT, &zi_noisedata.period, " step=1 ");

	TwAddVarRW(barPhysics, "Xg base", TW_TYPE_FLOAT, &xg_base, " step=0.001 ");
	TwAddVarRW(barPhysics, "Xg noise", noisecontroller, &xg_noise, NULL);
	TwAddVarRW(barPhysics, "Xg amp", TW_TYPE_FLOAT, &xg_noisedata.amplitude, " step=0.001 ");
	TwAddVarRW(barPhysics, "Xg period", TW_TYPE_FLOAT, &xg_noisedata.period, " step=1 ");
	TwAddVarRW(barPhysics, "Yg base", TW_TYPE_FLOAT, &yg_base, " step=0.001 ");
	TwAddVarRW(barPhysics, "Yg noise", noisecontroller, &yg_noise, NULL);
	TwAddVarRW(barPhysics, "Yg amp", TW_TYPE_FLOAT, &yg_noisedata.amplitude, " step=0.001 ");
	TwAddVarRW(barPhysics, "Yg period", TW_TYPE_FLOAT, &yg_noisedata.period, " step=1 ");
	TwAddVarRW(barPhysics, "Zg base", TW_TYPE_FLOAT, &zg_base, " step=0.001 ");
	TwAddVarRW(barPhysics, "Zg noise", noisecontroller, &zg_noise, NULL);
	TwAddVarRW(barPhysics, "Zg amp", TW_TYPE_FLOAT, &zg_noisedata.amplitude, " step=0.001 ");
	TwAddVarRW(barPhysics, "Zg period", TW_TYPE_FLOAT, &zg_noisedata.period, " step=1 ");
	TwAddSeparator(barPhysics,NULL,NULL);
	TwAddVarRW(barPhysics, "Xi damp", TW_TYPE_FLOAT, &dampx, " step=0.0001 ");
	TwAddVarRW(barPhysics, "Yi damp", TW_TYPE_FLOAT, &dampy, " step=0.0001 ");
	TwAddVarRW(barPhysics, "Zi damp", TW_TYPE_FLOAT, &dampz, " step=0.0001 ");

	TwAddVarRW(barEmitter, "Start X", TW_TYPE_FLOAT, &startx, " step=0.01 min=-100 max=100  ");
	TwAddVarRW(barEmitter, "Start Y", TW_TYPE_FLOAT, &starty, " step=0.01 min=0 max=100 ");
	TwAddVarRW(barEmitter, "Start Z", TW_TYPE_FLOAT, &startz, " step=0.01 min=-100 max=100  ");
	TwAddSeparator(barEmitter,NULL,NULL);
	TwAddVarRW(barEmitter, "Emitter X size", TW_TYPE_FLOAT, &xsize, " step=0.1 min=0 max=100  ");
	TwAddVarRW(barEmitter, "Emitter Y size", TW_TYPE_FLOAT, &ysize, " step=0.1 min=0 max=100 ");
	TwAddVarRW(barEmitter, "Emitter Z size", TW_TYPE_FLOAT, &zsize, " step=0.1 min=0 max=100  ");

	TwAddVarRW(barTexture, "Alpha start", TW_TYPE_FLOAT, &alpha0, " step=0.01 min=0 max=1  ");
	TwAddVarRW(barTexture, "Alpha end", TW_TYPE_FLOAT, &alpha1, " step=0.01 min=0 max=1  ");
	TwAddVarRW(barTexture, "Color start", TW_TYPE_COLOR3F, color0, " step=0.01 min=0 max=1  ");
	TwAddVarRW(barTexture, "Color end", TW_TYPE_COLOR3F, color1, " step=0.01 min=0 max=1  ");
	TwAddSeparator(barTexture,NULL,NULL);
	TwAddVarRW(barTexture, "Blend mode", blendmodes, &blendmode, NULL);
	TwAddVarRW(barTexture, "Depth writes", TW_TYPE_BOOL32, &zbufwrite, NULL);
	TwAddVarRO(barTexture, "Sort particles", TW_TYPE_BOOL32, &sorted, NULL);
	TwAddSeparator(barTexture,NULL,NULL);
	TwAddButton(barTexture, "Load texture...", loadtx, NULL, NULL);
	TwAddVarRO(barTexture, "Texture", TW_TYPE_CDSTRING, &texture_fn_short, NULL);
	TwAddVarRO(barTexture, "Frames", TW_TYPE_INT32, &tex_frames, NULL);
	TwAddButton(barTexture, "Reload texture", reloadtx, NULL, NULL);

	TwAddVarRW(barSprite, "Size start", TW_TYPE_FLOAT, &size0, " step=0.1 min=0 max=100  ");
	TwAddVarRW(barSprite, "Size end", TW_TYPE_FLOAT, &size1, " step=0.1 min=0 max=100  ");
	TwAddSeparator(barSprite,NULL,NULL);
	TwAddVarRW(barSprite, "R base", TW_TYPE_FLOAT, &r_base, " step=0.01 min=-6.28 max=6.28  ");
	TwAddVarRW(barSprite, "R noise", noisecontroller, &r_noise, NULL);
	TwAddVarRW(barSprite, "R amp", TW_TYPE_FLOAT, &r_noisedata.amplitude, " step=0.01 ");
	TwAddVarRW(barSprite, "R period", TW_TYPE_FLOAT, &r_noisedata.period, " step=1 ");
	TwAddVarRW(barSprite, "Ri base", TW_TYPE_FLOAT, &ri_base, " step=0.01 min=-6.28 max=6.28  ");
	TwAddVarRW(barSprite, "Ri noise", noisecontroller, &ri_noise, NULL);
	TwAddVarRW(barSprite, "Ri amp", TW_TYPE_FLOAT, &ri_noisedata.amplitude, " step=0.01 ");
	TwAddVarRW(barSprite, "Ri period", TW_TYPE_FLOAT, &ri_noisedata.period, " step=1 ");
}

int main (int argc, char** args)
{
    gVisualRand.init_genrand(0xc0cac01a);

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) 
    {
        fprintf(stderr, "Video initialization failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    initvideo(argc);

	nativecode_init(argc, args);

	TwInit(TW_OPENGL, NULL);

	TwWindowSize(gScreenWidth, gScreenHeight);

	initbars();

    tex_font = load_texture("fona.png");
	tex_particle = 0;//load_texture("sparkles.png");
		//load_texture("t1.png");
	tex_frames = 0;

	// For imgui - Enable keyboard repeat to make sliders more tolerable
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	// For imgui - Enable keyboard UNICODE processing for the text field.
	SDL_EnableUNICODE(1);
  
    // set window title
    SDL_WM_SetCaption(TITLE " - http://iki.fi/sol/", NULL);

    // hide cursor
//    SDL_ShowCursor(0);

//	SDL_SetCursor(load_cursor("cursor_scissors.png",0,0));
    
    while (1) 
    {
        process_events();
        draw_screen();
    }

    return 0;
}
